"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = void 0;
require = require("esm")(module);
const path_1 = require("path");
const rollup_1 = require("rollup");
const globby_1 = __importDefault(require("globby"));
const rollup_plugin_postcss_1 = __importDefault(require("rollup-plugin-postcss"));
const plugin_typescript_1 = __importDefault(require("@rollup/plugin-typescript"));
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_alias_1 = __importDefault(require("@rollup/plugin-alias"));
const document_1 = require("../document");
const compat_1 = __importDefault(require("preact/compat"));
const preact_render_to_string_1 = __importDefault(require("preact-render-to-string"));
const fs_1 = require("fs");
const { readdir, readFile, rmdir, writeFile, mkdir, copyFile, stat } = fs_1.promises;
const ROOT_DIR = path_1.join(process.cwd(), "src");
// const _vnode = options.vnode;
// options.vnode = vnode => {
//   if (vnode.type && (vnode.type as any).hydrate) {
//   }
//   if (_vnode) {
//     _vnode(vnode);
//   }
// }
const requiredPlugins = [
    plugin_alias_1.default({
        entries: [
            { find: /^@\/(.*)/, replacement: path_1.join(ROOT_DIR, "$1.js") },
            { find: "react/jsx-runtime", replacement: "preact/jsx-runtime" },
            { find: "react", replacement: "preact/compat" },
            { find: "react-dom", replacement: "preact/compat" },
        ],
    }),
    plugin_node_resolve_1.default({
        mainFields: ["module", "main"],
        dedupe: ["preact/compat"],
    }),
];
const globalPlugins = [
    rollup_plugin_postcss_1.default({
        config: true,
        inject: false,
        extract: true,
        minimize: true,
        sourceMap: false,
    }),
];
const createPagePlugins = () => [
    rollup_plugin_postcss_1.default({
        config: true,
        inject: false,
        extract: true,
        minimize: true,
        modules: {
            generateScopedName: "[hash:base64:5]",
        },
        sourceMap: false,
    }),
];
const OUTPUT_DIR = "./.tmp/microsite";
const outputOptions = {
    format: "esm",
    sourcemap: false,
};
const internalRollupConfig = {
    external: [
        "microsite/head",
        "microsite/document",
        "microsite",
        "preact/compat",
        "preact/jsx-runtime",
        "preact-render-to-string",
    ],
    treeshake: true,
    onwarn(message) {
        if (/empty chunk/.test(message))
            return;
        console.error(message);
    },
};
async function writeGlobal() {
    const global = await rollup_1.rollup(Object.assign(Object.assign({}, internalRollupConfig), { plugins: [
            ...requiredPlugins,
            plugin_typescript_1.default({ target: "ES2018" }),
            ...globalPlugins,
        ], input: "src/global.ts" }));
    const legacy = await rollup_1.rollup(Object.assign(Object.assign({}, internalRollupConfig), { plugins: [
            ...requiredPlugins,
            plugin_typescript_1.default({ target: "ES5" }),
            ...globalPlugins,
        ], input: "src/global.ts" }));
    try {
        return Promise.all([
            global.write({
                format: "esm",
                sourcemap: false,
                dir: OUTPUT_DIR,
                name: "global",
            }),
            legacy.write({
                format: "system",
                sourcemap: false,
                file: path_1.join(OUTPUT_DIR, "global.legacy.js"),
            }),
        ]);
    }
    catch (e) {
        console.log(e);
    }
}
async function writePages() {
    try {
        const pages = await globby_1.default(["src/pages/**/*.tsx"]);
        const bundles = await Promise.all(pages.map((input) => rollup_1.rollup(Object.assign(Object.assign({}, internalRollupConfig), { plugins: [
                ...requiredPlugins,
                plugin_typescript_1.default({ target: "ES2018" }),
                ...createPagePlugins(),
            ], input }))));
        const result = Promise.all(bundles.map((bundle, i) => bundle.write(Object.assign(Object.assign({}, outputOptions), { dir: pages[i]
                .replace(/^src/, OUTPUT_DIR)
                .split(path_1.sep)
                .slice(0, -1)
                .join(path_1.sep) }))));
        return result;
    }
    catch (e) {
        console.log(e);
    }
}
async function readDir(dir) {
    const entries = await readdir(dir, { withFileTypes: true });
    return Promise.all(entries.map((entry) => entry.isDirectory()
        ? readDir(path_1.join(dir, entry.name))
        : path_1.join(dir, entry.name))).then((arr) => arr.flat(Infinity));
}
async function prepare() {
    const paths = ["./dist", "./.tmp/microsite"];
    await Promise.all(paths.map((p) => rmdir(p, { recursive: true })));
    await Promise.all(paths.map((p) => mkdir(p, { recursive: true })));
    if ((await stat("./src/public")).isDirectory()) {
        const files = await readDir("./src/public");
        await Promise.all(files.map((file) => copyFile(path_1.resolve(process.cwd(), file), path_1.resolve(process.cwd(), "./dist/" + file.slice("src/public/".length)))));
    }
}
async function cleanup() {
    const paths = ["./.tmp/microsite"];
    await Promise.all(paths.map((p) => rmdir(p, { recursive: true })));
    if ((await readDir("./.tmp")).length === 0) {
        await rmdir("./.tmp");
    }
}
async function build() {
    await prepare();
    await Promise.all([writeGlobal(), writePages()]);
    const globalStyle = await readFile("./.tmp/microsite/global.css").then((v) => v.toString());
    const hasGlobalScript = await readFile("./.tmp/microsite/global.js").then((v) => !!v.toString().trim());
    if (hasGlobalScript) {
        await Promise.all([
            copyFile(path_1.resolve("./.tmp/microsite/global.js"), "dist/index.js"),
            copyFile(path_1.resolve("./.tmp/microsite/global.legacy.js"), "dist/index.legacy.js"),
        ]);
    }
    const files = await readDir("./.tmp/microsite/pages");
    const getName = (f) => f.slice(f.indexOf("pages/") + "pages/".length - 1, path_1.extname(f).length * -1);
    const styles = await Promise.all(files
        .filter((f) => f.endsWith(".css"))
        .map((f) => readFile(f).then((buff) => ({
        __name: getName(f),
        content: buff.toString(),
    }))));
    const pages = await Promise.all(files
        .filter((f) => f.endsWith(".js"))
        .map((f) => Promise.resolve().then(() => __importStar(require(path_1.join(process.cwd(), f)))).then((mod) => (Object.assign(Object.assign({}, mod), { __name: getName(f) })))));
    const output = [];
    for (const page of pages) {
        const { Page, __name } = page;
        const { content: style = null } = styles.find((style) => style.__name === __name) || {};
        try {
            output.push({
                name: __name,
                content: "<!DOCTYPE html>\n" +
                    preact_render_to_string_1.default(compat_1.default.createElement(document_1.Document, { render: preact_render_to_string_1.default, hasScripts: hasGlobalScript, styles: [globalStyle, style].filter((v) => v) },
                        compat_1.default.createElement(Page, null)), {}, { pretty: true }),
            });
        }
        catch (e) {
            console.log(`Error building ${__name}`);
            console.error(e);
            return;
        }
    }
    await Promise.all(output.map(({ name, content }) => mkdir(path_1.resolve(`./dist/${path_1.dirname(name)}`), { recursive: true }).then(() => writeFile(path_1.resolve(`./dist/${name}.html`), content))));
    await cleanup();
}
exports.build = build;
